@@ -1,3 +1,4 @@
+#include "../config.h"
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -14,13 +15,22 @@
 #include <linux/fd.h>
 #include <linux/fs.h>
 #include <linux/minix_fs.h>
+
+#ifdef HAVE_LINUX_EXT_FS_H
 #include <linux/ext_fs.h>
+#endif
+
 #include <linux/ext2_fs.h>
+
+#ifdef HAVE_LINUX_XIA_FS_H
 #include <linux/xia_fs.h>
+#endif
+
 #include <syslog.h>
 #include <sys/types.h>
 #include <grp.h>
 
+
 #define USE_2M
 #include "msdos_fs.h"
 
@@ -440,6 +450,7 @@ int id_fstype(byte *super, fmt_descr *fmt) {
 	    return T_EXT2;
 	}
 
+#ifdef EXT_SUPER_MAGIC
     /* look for ext filesystem */
 
     if (ext->s_magic==EXT_SUPER_MAGIC)
@@ -447,7 +458,9 @@ int id_fstype(byte *super, fmt_descr *fmt) {
 	    fmt->totsect=ext->s_nzones * (2<<ext->s_log_zone_size);
 	    return T_EXT;
 	}
+#endif
 
+#ifdef _XIAFS_SUPER_MAGIC
     /* look for xia filesystem */
 
     if (xia->s_magic==_XIAFS_SUPER_MAGIC)
@@ -455,7 +468,7 @@ int id_fstype(byte *super, fmt_descr *fmt) {
 	    fmt->totsect=xia->s_nzones * xia->s_zone_size/512;
 	    return T_XIA;
 	}
-
+#endif
     /* add more format types here ... */
 
     /* look for MS-DOG filesystem 
