@@ -16,6 +16,8 @@
 #include <linux/major.h>
 #include <linux/fs.h>
 #include <sys/time.h>
+#include <errno.h>
+#include "fdutils.h"
 
 #ifdef FD_RAW_MORE
 
@@ -47,7 +49,7 @@ char *error_msg[22]={
 
 enum dir { READ_, WRITE_ } ;
 
-static int curtrack = -1;
+static int curcylinder = -1;
 
 int interpret_errors(struct floppy_raw_cmd *raw_cmd, int probe_only)
 {
@@ -61,7 +63,7 @@ int interpret_errors(struct floppy_raw_cmd *raw_cmd, int probe_only)
 				case 0x40:
 					if(probe_only)
 						return -2;
-					curtrack = -1;
+					curcylinder = -1;
 					fprintf(stderr,
 						"\nerror during command execution\n");
 					if ( raw_cmd[k].reply[1] & ST1_WP ){
@@ -90,18 +92,18 @@ int interpret_errors(struct floppy_raw_cmd *raw_cmd, int probe_only)
 					sleep(4);
 					return k;
 				case 0x80:
-					curtrack = -1;
+					curcylinder = -1;
 					fprintf(stderr,
 						"\ninvalid command given\n");
 					return 1;
 				case 0xc0:
-					curtrack = -1;
+					curcylinder = -1;
 					fprintf(stderr,
 						"\nabnormal termination caused by polling\n");
 					return 0;
 				case 0:
 					if (raw_cmd[k].flags & FD_RAW_NEED_SEEK)
-						curtrack = raw_cmd[k].track;
+						curcylinder = raw_cmd[k].track;
 					/* OK */			
 			}
 		} else {
@@ -118,11 +120,11 @@ int send_cmd(int fd,struct floppy_raw_cmd *raw_cmd, char *message, int probe_onl
 	int i,j,k;
 	
 	for (j=0; j<4; j++){
-		if ( raw_cmd->track == curtrack && 
+		if ( raw_cmd->track == curcylinder && 
 		     !(raw_cmd->flags & FD_RAW_WRITE))
 			raw_cmd->flags &= ~FD_RAW_NEED_SEEK;
 		if ( ioctl( fd, FDRAWCMD, raw_cmd) < 0 ){
-			curtrack = -1;
+			curcylinder = -1;
 			if (errno == EBUSY){
 				i--;
 				fprintf(stderr,
@@ -159,7 +161,8 @@ int send_cmd(int fd,struct floppy_raw_cmd *raw_cmd, char *message, int probe_onl
 int readwrite_sectors(int fd, /* file descriptor */
 		      int drive,
 		      enum dir direction,
-		      int track, int head, int sector, int size, /* address */
+		      int cylinder, int head, 
+		      int sector, int size, /* address */
 		      char *data, 
 		      int bytes,
 		      struct floppy_raw_cmd *raw_cmd,
@@ -184,7 +187,7 @@ int readwrite_sectors(int fd, /* file descriptor */
 
     raw_cmd->cmd[1] = (drive & 3) | (head << 2);
     
-    raw_cmd->cmd[2] = track;
+    raw_cmd->cmd[2] = cylinder;
     raw_cmd->cmd[3] = head;
     raw_cmd->cmd[4] = sector;
     raw_cmd->cmd[5] = size;
@@ -193,7 +196,7 @@ int readwrite_sectors(int fd, /* file descriptor */
     raw_cmd->cmd[7] = 0x1b;
     raw_cmd->cmd[8] = 0xff;
 
-    raw_cmd->track = track;
+    raw_cmd->track = cylinder;
     return 0;
 }
 
@@ -305,7 +308,7 @@ static int hs=0;
 static int ts=14;
 static int debug=0;
 
-void format_track(int fd, int drive, int track, int head, struct xdf_struct *fmt,
+void format_track(int fd, int drive, int cylinder, int head, struct xdf_struct *fmt,
 		  int FatSize, int RootDirSize)
 {
 	int i,j;
@@ -314,22 +317,16 @@ void format_track(int fd, int drive, int track, int head, struct xdf_struct *fmt
 #define NEXT(x,max) x+=2; if(x>=max)x=1;
 	struct floppy_raw_cmd raw_cmd;
 	sector_map *p;
-	
-	struct format_map {
-		unsigned char track;
-		unsigned char head;
-		unsigned char sector;
-		unsigned char size;
-	} format_map[256];
+	format_map_t format_map[256];
 	
 	for(i=0; i<256; i++){
 		format_map[i].sector = i;
 		format_map[i].size = fmt->sizecode;
-		format_map[i].track = track;
+		format_map[i].cylinder = cylinder;
 		format_map[i].head = head;
 	}
 	
-	if ( track == 0 ){
+	if (cylinder == 0){
 		j = 0;
 		if(head) {
 			for (i=0; i< fmt->sect_per_track_1; i++) {
@@ -356,7 +353,7 @@ void format_track(int fd, int drive, int track, int head, struct xdf_struct *fmt
 		raw_cmd.cmd[2] = 2;
 
 	} else {
-		skew = (track * fmt->skew) % fmt->period;
+		skew = (cylinder * fmt->skew) % fmt->period;
 		max=0;
 		for(p = fmt->map; p->size; p++) {
 			if(p->head == head) {
@@ -382,7 +379,7 @@ void format_track(int fd, int drive, int track, int head, struct xdf_struct *fmt
 	raw_cmd.cmd[1] = (drive & 3 ) | ( head << 2 );
 	raw_cmd.cmd[5] = 42;
 	raw_cmd.cmd_count = 6;  
-	raw_cmd.track = track;
+	raw_cmd.track = cylinder;
 	send_cmd(fd, &raw_cmd, "format", 0);
 }
 
@@ -390,7 +387,7 @@ void format_track(int fd, int drive, int track, int head, struct xdf_struct *fmt
 enum fs { MAIN_FS, AUX_FS };
 
 void read_sectors(enum fs fs, int sector, char **data, int sectors,
-		  int drive, int fd, int track, struct xdf_struct *fmt, 
+		  int drive, int fd, int cylinder, struct xdf_struct *fmt, 
 		  int direction, struct floppy_raw_cmd *raw_cmd, int *j)
 {
 	int limit, head, psector, lsectors;
@@ -416,7 +413,7 @@ void read_sectors(enum fs fs, int sector, char **data, int sectors,
 				lsectors = 8 - sector;
 		}
 		readwrite_sectors(fd, drive, direction,
-				  track, head, psector, 
+				  cylinder, head, psector, 
 				  2, *data, lsectors << 9,
 				  raw_cmd + (*j)++, fmt->rate);
 		*data += lsectors << 9;
@@ -427,9 +424,9 @@ void read_sectors(enum fs fs, int sector, char **data, int sectors,
 
 #define WORD(x) ((unsigned char)(x)[0] + (((unsigned char)(x)[1]) << 8))
 
-void readwrite_track(int fd, int drive, enum dir direction, 
-		     int track, char *data, struct xdf_struct *fmt,
-		     int FatSize, int RootDirSize)
+void readwrite_cylinder(int fd, int drive, enum dir direction, 
+			int cylinder, char *data, struct xdf_struct *fmt,
+			int FatSize, int RootDirSize)
 {
     struct floppy_raw_cmd raw_cmd[20];
     int i,j;
@@ -438,11 +435,11 @@ void readwrite_track(int fd, int drive, enum dir direction,
     sector_map *map;
 
     j=i=0;
-    if(track) {
+    if(cylinder) {
 	    map = fmt->map;
 	    while(map->size){
 		    readwrite_sectors(fd, drive, direction,
-				      track, map->head, 
+				      cylinder, map->head, 
 				      map->size + 128, 
 				      map->size,
 				      data, 128 << map->size, 
@@ -454,25 +451,25 @@ void readwrite_track(int fd, int drive, enum dir direction,
     } else {
 	    /* the boot sector & the FAT */
 	    read_sectors(MAIN_FS, 0, &data, 1 + FatSize,
-			 drive, fd, track, fmt, direction, raw_cmd, &j);
+			 drive, fd, cylinder, fmt, direction, raw_cmd, &j);
 	    
 	    /* the index fs */
 	    read_sectors(AUX_FS, 0, &data, 8,
-			 drive, fd, track, fmt, direction, raw_cmd, &j);
+			 drive, fd, cylinder, fmt, direction, raw_cmd, &j);
 
 	    if( direction == READ_) {
 		    /* the remaining sectors of the phantom FAT */
 		    read_sectors(MAIN_FS, 9, &data, FatSize - 8,
-				 drive, fd, track, fmt, direction, raw_cmd, &j);
+				 drive, fd, cylinder, fmt, direction, raw_cmd, &j);
 	    } else
 		    data += (FatSize - 8) * 512;
 
 	    read_sectors(MAIN_FS, 1 + FatSize, & data, RootDirSize,
-			 drive, fd, track, fmt, direction, raw_cmd, &j);
+			 drive, fd, cylinder, fmt, direction, raw_cmd, &j);
 
 	    if (direction == READ_)
 		    read_sectors(AUX_FS, 3, &data, 5,
-				 drive,fd,track,fmt, direction, raw_cmd, &j);
+				 drive,fd,cylinder,fmt, direction, raw_cmd, &j);
 	    else
 		    data += 5 * 512;
 	    read_sectors(MAIN_FS,
@@ -480,7 +477,7 @@ void readwrite_track(int fd, int drive, enum dir direction,
 			 &data, 
 			 fmt->sect_per_track_any * 2 - RootDirSize -
 			 2 * FatSize - 6,
-			 drive, fd, track, fmt, direction, raw_cmd, &j);
+			 drive, fd, cylinder, fmt, direction, raw_cmd, &j);
     }
 
     raw_cmd[j-1].flags &= ~ FD_RAW_MORE;
@@ -489,11 +486,14 @@ void readwrite_track(int fd, int drive, enum dir direction,
     send_cmd(fd, raw_cmd, "read/write", 0);
     if(debug) {
 	    gettimeofday(&tv2,0);
-	    printf("\ntrack %d: %d %d %d\n\n", track,
-		   (tv2.tv_sec-tv1.tv_sec) * 1000000 + tv2.tv_usec - tv1.tv_usec,
-		   (tv2.tv_sec-tv1.tv_sec) * 1000000 + tv2.tv_usec - tv1.tv_usec,
-		   (tv2.tv_sec-tv4.tv_sec) * 1000000 + tv2.tv_usec - tv4.tv_usec);
-	    if(track == 2)
+	    printf("\ncylinder %d: %ld %ld %ld\n\n", cylinder,
+		   (long) ((tv2.tv_sec-tv1.tv_sec) * 1000000 + 
+			   tv2.tv_usec - tv1.tv_usec),
+		   (long) ((tv2.tv_sec-tv1.tv_sec) * 1000000 + 
+			   tv2.tv_usec - tv1.tv_usec),
+		   (long) ((tv2.tv_sec-tv4.tv_sec) * 1000000 + 
+			   tv2.tv_usec - tv4.tv_usec));
+	    if(cylinder == 2)
 		    tv4 = tv2;
 	    tv3 = tv2;
     }
@@ -524,20 +524,20 @@ static void usage(char *progname)
     fprintf(stderr,"Usage:\n");
     fprintf(stderr," For copying: %s [-n] <source> <target>\n", progname);
     fprintf(stderr,
-	    " For formatting: %s [-D dosdrive] [-t trackskew] [-h headskew] <target>\n",
+	    " For formatting: %s [-D dosdrive] [-t cylinderskew] [-h headskew] <target> [-01234]\n",
 	    progname);
     exit(1);
 }
 
 static char *readme=
 "This is an Xdf disk. To read it in Linux, you have to use a version of\r\n"
-"mtools which is more recent than 2.5.4, and set the environmental\r\n"
+"mtools which is more recent than 3.0, and set the environmental\r\n"
 "variable MTOOLS_USE_XDF before accessing it.\r\n\r\n"
 "Bourne shell syntax (sh, ash, bash, ksh, zsh etc):\r\n"
 " export MTOOLS_USE_XDF=1\r\n\r\n"
 "C shell syntax (csh and tcsh):\r\n"
 " setenv MTOOLS_USE_XDF 1\r\n\r\n"
-"mtools can be gotten from ftp://ftp.imag.fr/pub/Linux/ZLIBC/mtools\r\n"
+"mtools can be gotten from http://linux.wauug.org/pub/knaff/mtools\r\n"
 "\032";
 
 int progress;
@@ -553,7 +553,7 @@ int main(int argc, char **argv)
 {
     int sfd=0, tfd;
     int sdrive=0, tdrive;
-    int track, head, ret;
+    int cylinder, head, ret;
     int c;
     int noformat=0;
     char dosdrive=0;
@@ -680,7 +680,7 @@ int main(int argc, char **argv)
     }
 
     fmt = xdf_table + type;
-    for ( track = 0 ; track < max_cylinder ; track++){
+    for ( cylinder = 0 ; cylinder < max_cylinder ; cylinder++){
 	    if(sourcename) {
 		    if ( sdrive == - 1 ){
 			    ret = read(sfd, buffer, fmt->sect_per_track_any*1024);
@@ -694,16 +694,16 @@ int main(int argc, char **argv)
 			    }
 		    } else {
 			    if(progress) {
-				    fprintf(stderr,"r%02d", track);
+				    fprintf(stderr,"r%02d", cylinder);
 				    fflush(stderr);
 			    }
-			    readwrite_track(sfd, sdrive, READ_, track, buffer, fmt,
+			    readwrite_cylinder(sfd, sdrive, READ_, cylinder, buffer, fmt,
 					    FatSize, RootDirSize);
 			    clear();
 		    }
 	    } else {
 		    memset(buffer, 0, fmt->sect_per_track_any * 1024);
-		    if(!track) {
+		    if(!cylinder) {
 			    buffer[17]=0xe0;
 			    buffer[18]=0;
 			    buffer[22]=FatSize;
@@ -726,20 +726,20 @@ int main(int argc, char **argv)
 	    } else {
 		    if(!noformat) {
 			    if(progress) {
-				    fprintf(stderr,"f%02d", track);
+				    fprintf(stderr,"f%02d", cylinder);
 				    fflush(stderr);
 			    }
 			    for(head = 0; head < 2; head++)
-				    format_track(tfd, tdrive, track,head, fmt,
+				    format_track(tfd, tdrive, cylinder,head, fmt,
 						 FatSize, RootDirSize);
 			    clear();
 		    }
 		    if(progress) {
-			    fprintf(stderr,"w%02d", track);
+			    fprintf(stderr,"w%02d", cylinder);
 			    fflush(stderr);
 		    }
-		    readwrite_track(tfd, tdrive, WRITE_, track, buffer, fmt,
-				    FatSize, RootDirSize);
+		    readwrite_cylinder(tfd, tdrive, WRITE_, cylinder, buffer, 
+				       fmt, FatSize, RootDirSize);
 		    clear();
 	    }
     }
